#include	"types.h"
#include	"item.h"
#include	"mem.h"
#include	"nodes.h"
#include	"debug.h"
#include	"list.h"
#include	"device.h"
#include	"driver.h"
#include	"kernel.h"
#include	"kernelnodes.h"
#include	"io.h"

#include	"operror.h"

#include	"filesystem.h"
#include	"filesystemdefs.h"
#include	"discdata.h"

#include	"directory.h"
#include 	"directoryfunctions.h"

#include	"filefunctions.h"

#include	"strings.h"

#include	"baseUtils.h"

#define	ERRORCODE	0
#define	OK			1

#define	bufsize(buf)	(strlen(buf)+1)

#define	isFile(de)	!(de.de_Flags & FILE_IS_DIRECTORY)

#define	BUFFSIZE		256

char	buff[BUFFSIZE];

long	startTime,endTime;

void
PrintfSysErr(Item);

/*
	changeDir
				Parameters:		path:	string specifying the relative or absolute pathname of the new directory.
				Return Value:	OK			(TBD)
								ERRORCODE	(TBD)
								
				The following procedure changes the current directory to the specified directory in the 
				path parameter.  By using the GetDirectory call, it notifies the user of the task's working directory
				before and after the call to ChangeDirectory.
					
*/

int
changeDir(char *path)
{
	Item		fileItem1;							/*	Items returned by calls to ChangeDirectory 	*/
	ItemNodeP	nodeP;

	if (strcmp(path,"/cd-rom") == 0)
	{
		kprintf("Feature not implemented\n");
		return OK;
	}

	startTime = TICKCOUNT();
	fileItem1 = ChangeDirectory(path);
	endTime = TICKCOUNT();
	
	if (fileItem1 < 0){
		GetSysErr(buff,BUFFSIZE,fileItem1);
		kprintf("FAILED ChangeDirectory() CALL\n%s\n",buff);
		return ERRORCODE;
	}
	else{
		nodeP = (ItemNodeP)LookupItem(fileItem1);
		if (nodeP)
			pipeData("New Directory:  %s\n",nodeP->n_Name);
		else{
			GetSysErr(buff, BUFFSIZE,fileItem1);
			pipeData("FAILED LookupItem Call\n%s\n",buff);
		}		
	}
	return (OK);
}

/*
	changeDirTest
				Parameters:		<NONE>
				Return Value:	<NONE>
								
				The following procedure calls changeDir() a number of times to change the working directory
				of this task.  
				
				TO DO:
					Modify to read the target paths from a file when writing test tool for reading from files.
					
*/
void
changeDirTest(void)
{
	int cases = 23,
		caseNum,
		callOK = OK;
	char *dirs[] = {"/remote",
					"abcdefghijklmnopqrstuvwxyzabcdef",		/*	32 character filename.  Illegal */
					"abcdefghijklmnopqrstuvwxyzabcde",			/*	31 character filename.  Legal.	*/
					"/remote",									/*	switch back to remote directory.	*/
					"abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde/abcdefghijklmnopqrstuvwxyzabcde"
															/*	^^^^^^^  255 character path name. 7 levels deep	*/
					".",									/*	Refers to current directory.  Working directory should not change.	*/
					"..",										/*	Refers to directory containing the working directory.	*/
					"^",										/*	Refers to the root directory.	*/
					"/remote/../remote",						/*	Should return to this /remote directory.	*/
					"./../..",									/*	Changes current directory to "grandparent" directory.	*/
					"/..",										/*	Changes working directory to directory containing root directory.	*/
					"/nvram",									/*	nvram directory	*/
					"/cd-rom",									/*	Not implemented yet	*/
					"/remote",									/*	*/
					"AppStartup",								/*	FileName.	Should return an error message.	*/
					"1/2/3/4/5/6/7/8/9/10",					/*	relative reference	*/
					"/remote/1/2/3/4/5/6/7/8/9/10",			/*	absolute reference	*/
					"1",										/*	1 level reference	*/
					"1/2",										/*	2 level reference	*/
					"1/2/3/4/5",								/*	5 level reference	*/
					"1/2/3/4/5/6/7/8/9/10",					/*	10 level reference	*/
					"1/2/3/4/5/6/7/8/9/10/1/2/3/4/5/6/7/8/9/10",							/*	20 level reference	*/
					"1/2/3/4/5/6/7/8/9/10/1/2/3/4/5/6/7/8/9/10/1/2/3/4/5/6/7/8/9/10"};		/*	30 level reference	changeDir("/remote");	*/
		
	for (caseNum = 0; caseNum < cases; caseNum++)
	{
		pipeData("TARGET DIRECTORY:  %s\n",dirs[caseNum]);
		startTime = TICKCOUNT();
		callOK = changeDir(dirs[caseNum]);
		endTime = TICKCOUNT();
		pipeData("Status: %s\tTime:  %d\n",
				callOK ? "PASS" : "FAIL", endTime - startTime);

	}	
}
