/*
	File:		ChangeDir.c

	Contains:	Test routine for the ChangeDirectory call.

	Written by:	Manuel Tomagan and Eric Hegstrom

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):
		 <3>	 12/21/93	ELH		integrate ChangeDirectory test into Directory Test tool
		 <2>	 8/3/93		CDE		Added duration logic.
									Fixed pipeData(kOuterTimingStr to print correct final iteration value
		 <1>	 7/10/93	CDE		Derived from TestShell.c

	To Do:
*/


/***** Checklist:

	 Change the "File" line above to reflect the name of your call
	 Change the "Contains" line above to reflect the call being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Increment the change number
		- Specify the date, your initials, and a brief description of the change(s)
		- When you first change this file, add "<2> (date) (initials) Derived from shell"
	 Look for bulleted () HINTS below.
*/

/* HINTS:

	 Change to reflect your call name.
*/
//#include "ChangeDir.h"
#include "baseUtils.h"

/* Actual call name */
#define		kCallNameStr	"ChangeDirectory"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s:\n"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i\n"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i\n"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s (%i)\n"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i\n"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s\n"

/**** Test routine macros ****/
/* HINTS:

	 See TestChangeDirectory below for usage.
*/
#define		PIPE_TIMING_HEADER		pipeData(kTimingHeaderStr,callName)
#define		PIPE_OUTER_TIMING		pipeData(kOuterTimingStr,minOuterIterations,lastIteration, accumulator)
#define		INNER_ITERATIONS		CallChangeDirectory(indx, gMinInnerIterations, gMaxInnerIterations)
/* HINTS:

	 Change to use your call name and add the call result and parameters to the parameter list.
*/
	
extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern long gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Duration */
extern long gDuration;
extern Boolean gUseDuration;

/* Automatic/interactive */
extern Boolean gAutomatic;

extern	char *gpathName;
extern	int	changeDir(char *);

/***** Test routines *****/
Boolean		VerifyChangeDirectory ( Boolean beforeCall, long aIndx )
/* Functionality verification for ChangeDirectory */
/* HINTS:

	 In all likelihood, parameter list and verification format will change drastically depending on the call
	 It may be desirable to use static variables for pre- and post- call comparison
*/
{
	
	if (beforeCall == kBeforeCall)
	{		
		if ( gVerify == kReport ) pipeData(kVerifyStr, "Before", aIndx);
		return true;
	}
	else
	{
		if ( gVerify == kReport ) pipeData(kVerifyStr, "After", aIndx);
		return true;
		/* HINTS:
		
			 Return true if the results are as expected, false otherwise
		*/
	}
	
}

long	CallChangeDirectory (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	/* HINTS:
	
		 You probably want to use an int32 (?) error value (see "Error-checking" below)
	*/
	
  	/* Variables for call parameters and return value */
	/* HINTS:
	
		 Number and type of local variables depend on the call and verification requirements.
	*/
	
	/* Functionality verification (pre-call) */
	if ( gVerify != kNoVerify ) theError = !VerifyChangeDirectory ( kBeforeCall, minInnerIterations );
	/* HINTS:
	
		 Call name, return value, and additional parameters depend on the call
			and verification requirements.
	*/

	/* Preparation for call iterations */
	theError = FALSE;
	/* HINTS:
	
		 Do whatever initialization is required
		 Change the above statement to get the real error value (possibly an int32)
	*/
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			pipeData(kIncidentalErrorStr, "ChangeDirectory");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
#if 0
		startTime = TICKCOUNT(); 
		
		kprintf("CALLING changeDir\n");
#endif		
		changeDirTest();
#if 0		
		endTime = TICKCOUNT();
		accumulator += endTime - startTime;
#endif
		/* Functionality verification (post-call) */
		if ( gVerify != kNoVerify ) theError = !VerifyChangeDirectory ( kAfterCall, indx );
		/* HINTS:
		
			 Call name, return value, and additional parameters depend on the call
				and verification requirements.
		*/
		
		/* Error-checking */
		if (gCheckErrors != kNone)
		{
			if (theError)
			{
				pipeData(kCallErrorStr, indx);
				if (gCheckErrors == kStop) return kBreakTime;
			}
		}
 	}
#if 0
	if (gOutputTiming)
		pipeData(kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
#endif	
	return accumulator;
}

void TestChangeDirectory( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
	long	startTime;
	long	elapsedTime;
	long	duration = SecondsToAudioTicks( gDuration );
	Boolean keepGoing = FALSE;
	long	lastIteration = 0;
		
	callName = kCallNameStr;
	STATUS(callName);
	if (gOutputTiming)
		PIPE_TIMING_HEADER;

	startTime = TICKCOUNT();

	do
	{
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			lastIteration = indx;
			
			theTime = INNER_ITERATIONS;

			accumulator += theTime;
			if (theTime == kBreakTime) break;
			
			if ( gUseDuration )
			{
				elapsedTime = TICKCOUNT() - startTime;
				if ( elapsedTime >= duration ) break;
			}
		}
	
		if (gOutputTiming)
			PIPE_OUTER_TIMING;
			
		if ( gUseDuration )
			keepGoing = ( elapsedTime < duration );
			
	} while ( keepGoing );
	
}


