/*
	File:		CreateAlias.c

	Contains:	Code to test CreateAlias routine.

	Written by:	Manuel Tomagan

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 12/01/93	MT		Derived from TestCallA.c

	CASES:
		 Directories
			- Absolute paths
			- Relative paths
			- Root
		 Files
		 Non-existent files for real path.
		 Special characters:  ^ . .. / $
		 Alias token equals real path
		 NULL real path
		 NULL alias path
		 '/' delimiters in alias name.
		 Case-sensitive?

	To Do:
*/


/***** Checklist:

	 Change the "File" line above to reflect the name of your call
	 Change the "Contains" line above to reflect the call being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Increment the change number
		- Specify the date, your initials, and a brief description of the change(s)
		- When you first change this file, add "<2> (date) (initials) Derived from shell"
*/

/* HINTS:

	 Change to reflect your call name.
*/
#include "FileTest.h"
#include "directory.h"
#include "directoryfunctions.h"
#include "data.h"
#include "nodes.h"
#include "filesystem.h"
#include "item.h"

/* Actual call name */
#define	kCallNameStr	"CreateAlias"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s:\n"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i\n"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i\n"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s (%i)\n"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i\n, path: %s\t\talias:  %s\n"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s\n"

/**** Test routine macros ****/
/* HINTS:

	 See TestCallA below for usage.
*/
#define		PIPE_TIMING_HEADER		pipeData(kTimingHeaderStr,callName)
#define		PIPE_OUTER_TIMING		pipeData(kOuterTimingStr,minOuterIterations,lastIteration, accumulator)
#define		INNER_ITERATIONS		CallCreateAlias(indx, gMinInnerIterations, gMaxInnerIterations)
/* HINTS:

	 Change to use your call name and add the call result and parameters to the parameter list.
*/
	
extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern long gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Duration */
extern long gDuration;
extern Boolean gUseDuration;

/* Automatic/interactive */
extern Boolean gAutomatic;

extern	char *gpathName;
extern	int	changeDir(char *);

/***** Test routines *****/

Boolean		VerifyCreateAlias ( Boolean beforeCall, char *aliasName, char *pathName, Item aliasItem )
/* Functionality verification for CreateAlias */
/* NOTES:
	
		Verification routines consists of looking up item returned by call and checking
			a)  System Type:  FILEFOLIO
			b)	Type:			FILEALIASNODE
			c)	Name of Item.
		
		Assumptions:
			OpenDiskFile returns a negative item number if the given path does not exist.
			
*/
{
//	Item		expItem;
	ItemNode	*ptr;
	Item		pathItem;
//	Boolean		result;
	
	if (beforeCall == kBeforeCall)
	{
		if (gCheckErrors != kStop)
			pipeData("ALIAS:  \"%s\" <==>  FILE:  \"%s\"\t\t\n",  aliasName, pathName);
		return TRUE;
	}	
	pathItem = OpenDiskFile(pathName);
	
	if (pathItem < 0){
		if (gCheckErrors != kStop)
			if (aliasItem >= 0)
				pipeData("ERROR.  EXPECTED NEGATIVE ITEM NUMBER.  PATH DOES NOT EXIST.\n");
		return (aliasItem < 0);
	}
				
	ptr = (ItemNode *)LookupItem(aliasItem);

	if (ptr == NULL){
		if ((gCheckErrors != kStop)
			&& (aliasName != NULL))
			pipeData("ERROR.  COULD NOT LOCATE aliasItem  (%i)\n", aliasItem);
		return (aliasName == NULL);
	}

	if (gCheckErrors != kStop){
		if (strcmp(ptr->n_Name, aliasName))
			pipeData("ERROR.  UNEXPECTED NAME:  %s, %s\n", aliasName, ptr->n_Name);
		else
			if (ptr->n_Type != FILEALIASNODE)
				pipeData("ERROR.  INCORRECT NODE TYPE:  %i\n", ptr->n_Type);
			else
				if (ptr->n_SubsysType != FILEFOLIO)
					pipeData("ERROR.  INCORRECT SUBSYSTYPE:  %i\n", ptr->n_SubsysType);
	}
	
	return (!strcmp(ptr->n_Name, aliasName)
			&& (ptr->n_Type == FILEALIASNODE)
			&& (ptr->n_SubsysType == FILEFOLIO));
}

long	CallCreateAlias (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	int32	indx1 = 0;

	Item	fileItem;
	


	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			pipeData(kIncidentalErrorStr, kCallNameStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		
		for ( indx1 = 0; indx1 < NUM_ALIASES; indx1++ )
		{
			if ( gVerify != kNoVerify )
				theError = !VerifyCreateAlias( kBeforeCall, aliases[indx1], realPaths[indx1], 0 );
			
			startTime = TICKCOUNT(); 
			
			fileItem = CreateAlias( aliases[ indx1 ], realPaths[indx1] );

			endTime = TICKCOUNT();

			accumulator += endTime - startTime;

			if (gVerify != kNoVerify)
			{
				theError = !VerifyCreateAlias( kAfterCall, aliases[indx1], realPaths[indx1],fileItem );
				if (gCheckErrors != kStop)
					if (theError)
					{
						pipeData(kCallErrorStr, indx, realPaths[indx1], aliases[indx1]);
						if (gCheckErrors == kStop)	
							return accumulator;
					}
			}

		}
		
 	}

	if (gOutputTiming)
		pipeData(kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);

	return accumulator;
}

void TestCreateAlias( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
	long	startTime;
	long	elapsedTime;
	long	duration = SecondsToAudioTicks( gDuration );
	Boolean keepGoing = FALSE;
	long	lastIteration = 0;
		
	callName = kCallNameStr;
	STATUS(callName);
	if (gOutputTiming)
		PIPE_TIMING_HEADER;

	startTime = TICKCOUNT();

	do
	{
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			lastIteration = indx;
			
			theTime = INNER_ITERATIONS;

			accumulator += theTime;
			if (theTime == kBreakTime) break;
			
			if ( gUseDuration )
			{
				elapsedTime = TICKCOUNT() - startTime;
				if ( elapsedTime >= duration ) break;
			}
		}
	
		if (gOutputTiming)
			PIPE_OUTER_TIMING;
			
		if ( gUseDuration )
			keepGoing = ( elapsedTime < duration );
			
	} while ( keepGoing );
	
}


