/*
	File:		LoadProgram.c

	Contains:	Code to test LoadProgram routine.

	Written by:	Eric Hegstrom

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <0>	 1/7/94	ELH		Derived from ReadDiskStream.c

	CASES TESTED:

	To Do:
	

*/


/***** Checklist:

	 Change the "File" line above to reflect the name of your call
	 Change the "Contains" line above to reflect the call being tested
	 Change the name in the "Written by" line above to your name
	 Add Change History lines above the previous lines:
		- Increment the change number
		- Specify the date, your initials, and a brief description of the change(s)
		- When you first change this file, add "<2> (date) (initials) Derived from shell"
*/

/* HINTS:

	 Change to reflect your call name.
*/
#include "FileTest.h"
#include "directory.h"
#include "directoryfunctions.h"
#include "data.h"
#include "item.h"

/* Actual call name */
#define	kCallNameStr	"LoadProgramTest"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s:\n"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i\n"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i\n"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s %s %s\n"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error Opening \"%s\" on iteration %i.  Returned Item number:  %i.\n"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s\n"

/**** Test routine macros ****/
/* HINTS:

	 See TestCallA below for usage.
*/
#define		PIPE_TIMING_HEADER		pipeData(kTimingHeaderStr,callName)
#define		PIPE_OUTER_TIMING		pipeData(kOuterTimingStr,minOuterIterations,lastIteration, accumulator)
#define		INNER_ITERATIONS		CallLoadProgram(indx, gMinInnerIterations, gMaxInnerIterations)
/* HINTS:

	 Change to use your call name and add the call result and parameters to the parameter list.
*/
	
extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern long gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Duration */
extern long gDuration;
extern Boolean gUseDuration;

/* Automatic/interactive */
extern Boolean gAutomatic;

extern	char *gpathName;
extern	int	changeDir(char *);

/***** Test routines *****/
Boolean		VerifyLoadProgram ( Boolean beforeCall, void *data )
/* Functionality verification for CallA */

/*

				Assumptions:
*/
{	
	return FALSE;

}

long	CallLoadProgram (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	int32	fileIndx = 0;

	Item	proItem;
	
//	Stream	*stream1;
	
//	int32	seekRslt = 0;
	int32	err = 0;
	


	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			pipeData(kIncidentalErrorStr, "CallA");
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		
		for ( fileIndx = 0; fileIndx < 1; fileIndx++ )
		{

		//	if ( gVerify != kNoVerify ) theError = !VerifyLoadProgram ( kBeforeCall, (void *)files[fileIndx] );

			startTime = TICKCOUNT(); 
			
			
			proItem = LoadProgram( "/remote/eventJoy");
			if (proItem < 0)
				pipeData("Error in LoadProgram( '/remote/eventJoy') returns %d \n", proItem);
			pipeData("Press start button on joy pad TWICE to continue \n");
			err = SetItemPri( proItem, 199);
			Yield();
			
			// fileItem = LoadProgram( files[ fileIndx ] );

			endTime = TICKCOUNT();

			if ( gVerify != kNoVerify )
			{
				theError = !VerifyLoadProgram ( kAfterCall, (void *)&proItem );

				/* Error-checking */
				if (gCheckErrors != kNone)
					if (theError)
					{
						pipeData(kCallErrorStr, files[fileIndx], indx, proItem);
						if (gCheckErrors == kStop)
							return kBreakTime;
					}
			}
			accumulator += endTime - startTime;
		}

 	}
#if 0
	if (gOutputTiming)
		pipeData(kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
#endif	
	return accumulator;
}

void TestLoadProgram( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
	long	startTime;
	long	elapsedTime;
	long	duration = SecondsToAudioTicks( gDuration );
	Boolean keepGoing = FALSE;
	long	lastIteration = 0;
		
	callName = kCallNameStr;
	STATUS(callName);
	if (gOutputTiming)
		PIPE_TIMING_HEADER;

	startTime = TICKCOUNT();

	do
	{
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			lastIteration = indx;
			
			theTime = INNER_ITERATIONS;

			accumulator += theTime;
			if (theTime == kBreakTime) break;
			
			if ( gUseDuration )
			{
				elapsedTime = TICKCOUNT() - startTime;
				if ( elapsedTime >= duration ) break;
			}
		}
	
		if (gOutputTiming)
			PIPE_OUTER_TIMING;
			
		if ( gUseDuration )
			keepGoing = ( elapsedTime < duration );
			
	} while ( keepGoing );
	
}


