/*
	File:		OpenDiskFileInDir.c

	Contains:	Code to test OpenDiskFileInDir routine.

	Written by:	Manuel Tomagan

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 12/01/93	MT		Derived from TestCallA.c

	To Do:
	
		1)	Define Verification routine
		2)	Implemented as "Sandwich".  Non-Sandwich implementation allows for testing of
				 Opening of opened files
				 Closing of closed files
				
	CASES TESTED:
		  Negative Directory Items
		  Non-existent files
		  NULL file paths
		  absolute file paths
		  relative file paths
		  alias
*/


/* HINTS:

	 Change to reflect your call name.
*/
#include "FileTest.h"
#include "directory.h"
#include "directoryfunctions.h"
#include "data.h"

/* Actual call name */
#define	kCallNameStr	"OpenDiskFileInDir"

/***** Reporting formats *****/
/* Timing output header format: (Call name):" */
#define		kTimingHeaderStr	"%s:\n"

/* Outer iteration timing output format: (outerFrom), (outerTo), (total time) */
#define		kOuterTimingStr		"\t%i\t%i\t%i\n"

/* Inner iteration timing output format: (innerFrom), (innerTo), (total time) */
#define		kInnerTimingStr		"\t%i\t%i\t%i\n"

/* Verification output format: (Before/after), (call iteration index) */
#define		kVerifyStr		"%s (%i)\n"

/* Call error reporting format: (call iteration index) */
#define		kCallErrorStr	"Error on iteration %i\n"

/* Incidental error reporting format: (call name) */
#define		kIncidentalErrorStr	"Error initializing for %s\n"

/**** Test routine macros ****/
/* HINTS:

	 See TestCallA below for usage.
*/
#define		PIPE_TIMING_HEADER		pipeData(kTimingHeaderStr,callName)
#define		PIPE_OUTER_TIMING		pipeData(kOuterTimingStr,minOuterIterations,lastIteration, accumulator)
#define		INNER_ITERATIONS		CallOpenDiskFileInDir(indx, gMinInnerIterations, gMaxInnerIterations)
/* HINTS:

	 Change to use your call name and add the call result and parameters to the parameter list.
*/
	
extern long gMinInnerIterations;
extern long gMaxInnerIterations;

/* error-checking */
extern long gCheckErrors;
extern long gCheckIncidentalErrors;

/* functionality verification*/
extern long gVerify;

/* timing output */
extern Boolean gOutputTiming;

/* fault checking*/
extern Boolean gCheckFault;

/* Increments */
extern long gIncrement1;

/* Duration */
extern long gDuration;
extern Boolean gUseDuration;

/* Automatic/interactive */
extern Boolean gAutomatic;

extern	char *gpathName;
extern	int	changeDir(char *);

/***** Test routines *****/
Boolean		VerifyOpenDiskFileInDir ( Boolean beforeCall, char *dirPath, void *data )
/* HINTS:

	 In all likelihood, parameter list and verification format will change drastically depending on the call
	 It may be desirable to use static variables for pre- and post- call comparison
*/
{
	static	char	pathcopy[256] = "";
	Item			expItem;
	ItemNodeP		ptr;
	int32			i;
	
	if (beforeCall == kBeforeCall)
	{		
		memcpy(pathcopy,(char *)data, strlen((char *)data)+1);
		if (gCheckErrors != kNone)
			pipeData("DIRECTORY:  \"%s\"\t\tFILE:  \"%s\"\n",	dirPath, (char *)data);
			
		if (!strcmp(pathcopy,".")
			|| !strcmp(pathcopy, ""))
			GetDirectory(pathcopy, 256);
		else
			if (!strcmp(pathcopy, ".."))
			{
				GetDirectory(pathcopy, 256);
				i = strlen(pathcopy);
				while (i > 0 && (pathcopy[i] != '/'))
					i--;
				if (pathcopy[i] == '/')
					pathcopy[i] = 0;
			}
		return TRUE;
	}
	expItem = *(Item *)data;
	if (expItem >= 0)
	{
		ptr = (ItemNodeP)LookupItem(expItem);
		
		if (gCheckErrors != kNone)
			pipeData("%s\n",ptr == NULL ? "NON-EXISTENT": "EXISTS" );
					
		if (ptr == NULL)
			return FALSE;

		i = strlen((char *)pathcopy);
		while ((i > 0) && (pathcopy[i] != '/'))
			i--;
		
		i += pathcopy[i] == '/';
		return !strcmp(pathcopy+i, ptr->n_Name)	&& (ptr->n_SubsysType == FILESYSTEMNODE);
	}
	if (gCheckErrors != kNone)
		pipeData("%s\n", "NON-EXISTENT");
	return TRUE;
	
}

long	CallOpenDiskFileInDir (  long currentOuterIteration, long minInnerIterations, long maxInnerIterations )
/* Inner test */
{
  	long	startTime = 0, endTime = 0, indx = 0, accumulator = 0;
	Boolean theError = FALSE;
	int32	dirIndx;
	Item	fileItem,
			dirItem;
	
	/* Incidental error-checking */
	if (gCheckIncidentalErrors != kNone)
	{
		if (theError)
		{
			pipeData(kIncidentalErrorStr, kCallNameStr);
			if (gCheckIncidentalErrors == kStop) return kBreakTime;
		}
	}

	/* Call iterations */
	for ( indx = minInnerIterations;indx <= maxInnerIterations; indx++ )
	{
		for ( dirIndx = 0; dirIndx < NUM_CASES; dirIndx++ )
		{
			if ( gVerify != kNoVerify ) 
				theError = !VerifyOpenDiskFileInDir ( kBeforeCall, dirs[dirIndx], files[dirIndx] );
		
			dirItem = OpenDiskFile( dirs[ dirIndx ] );
			
			startTime = TICKCOUNT(); 
	
			fileItem = OpenDiskFileInDir( dirItem, files[ dirIndx ] );
			
			endTime = TICKCOUNT();

			accumulator += endTime - startTime;
		
			if (gVerify != kNoVerify)
			{
				theError = !VerifyOpenDiskFileInDir ( kAfterCall, NULL, &fileItem );
				if (gCheckErrors != kNone)
					if (theError)
					{
						pipeData("Error DIR:  \"%s\", FILE:  \"%s\"\n",
									dirs[dirIndx], files[dirIndx]);
						if (gCheckErrors == kStop)
							return kBreakTime;
					}
			}
			if (fileItem >= 0)
				CloseDiskFile(fileItem);
			
			if (dirItem >= 0)
				CloseDiskFile(dirItem);
			
		}
		
#if 0
		/* Functionality verification (post-call) */
		if ( gVerify != kNoVerify ) theError = !VerifyCall ( kAfterCall, indx );
#endif
 	}
#if 0
	if (gOutputTiming)
		pipeData(kInnerTimingStr, minInnerIterations, maxInnerIterations, accumulator);
#endif	
	return accumulator;
}

void TestOpenDiskFileInDir( long minOuterIterations, long maxOuterIterations )
/* Outer test */
{
  	long	theTime = 0, indx = 0, accumulator = 0;
	char	*callName;
	long	startTime;
	long	elapsedTime;
	long	duration = SecondsToAudioTicks( gDuration );
	Boolean keepGoing = FALSE;
	long	lastIteration = 0;
		
	callName = kCallNameStr;
	STATUS(callName);
	if (gOutputTiming)
		PIPE_TIMING_HEADER;

	startTime = TICKCOUNT();

	do
	{
		for ( indx = minOuterIterations; indx <= maxOuterIterations; indx+= gIncrement1)
		{
			lastIteration = indx;
			
			theTime = INNER_ITERATIONS;

			accumulator += theTime;
			if (theTime == kBreakTime) break;
			
			if ( gUseDuration )
			{
				elapsedTime = TICKCOUNT() - startTime;
				if ( elapsedTime >= duration ) break;
			}
		}
	
		if (gOutputTiming)
			PIPE_OUTER_TIMING;
			
		if ( gUseDuration )
			keepGoing = ( elapsedTime < duration );
			
	} while ( keepGoing );
	
}


