 
/*
	File:		baseUtils.c

	Contains:	Base utilities for folio call tests

	Written by:	Robert Chinn, Charlie Eckhaus

	Copyright:	 1993 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <3>	 7/22/93	CDE		Added isNumber, HexToInt32, isHexNumber, GetNextNumParam,
		 							GetNextHexParam
									Changed GetFlag to GetNextFlagParam
		 <2>	 6/29/93	CDE		General fixup per Robert (new comments, deletion of unused code)
		 							Added gInvalidArg, command line argument validation
		 <1>	 6/22/93	CDE		Prepared for public consumption

	To Do:
*/

#include "baseUtils.h"

extern	Boolean gUseLogFile;
extern	FILE	*gTheFile;
extern	Boolean gInvalidArg;


long	power(long x, long y)
/* Return x to the yth power */
{
	 long i, p;
	 
	 p = 1;
	 for ( i = 1; i <= y; ++i)
	 	p = p * x;
		
	return p;
}


void 	printd(int32 n, FILE *theFile)
/* Used by pipedata to output decimal values to the log file */
{
	if (n/10)
		printd(n/10,theFile);
	 putc((char) n%10 + '0',theFile);
}


void 	printHex(int32 n, FILE *theFile)
/* Used by pipedata to output hex values to the log file */
{
	long	indx;
	char	*hexChar[] = {	"0","1","2","3","4",
							"5","6","7","8","9",
							"a","b","c","d","e","f" };
	
	for (indx = 7; indx >= 0; indx--)
	{
		fputs( hexChar[((n) >> indx * 4) & 0xF], theFile);	
	}
}


void	pipeData(char *formatStr,...)
/* Used like printf to output to the log file if it's being used,
	or to standard output otherwise */
{
	va_list		ap;
	char		*p,*sval,schar;
	int32		ival;
	
	va_start(ap, formatStr);
	for ( p = formatStr ; *p ; p++ )
	{
		if (*p == 0x0A) // look explicitly for a newline hex 10 or '\n'
		{
			if (!gUseLogFile) kprintf("\n");
			else putc(0x0D, gTheFile);
			
			continue;
		}
		if (*p != '%')
		{
			if (!gUseLogFile) kprintf("%c",*p);
			else putc(*p, gTheFile);
			
			continue;
		}
		switch(*++p)
		{
			case 'd':
			case 'i':
			case 'u':
				ival = va_arg(ap, int32);
				if (!gUseLogFile) kprintf("%i",ival);
				else printd(ival,gTheFile);
				break;
				
			case 'x':
				ival = va_arg(ap, int32);
				if (!gUseLogFile) kprintf("%08lx",ival);
				else printHex(ival,gTheFile);
				break;
				
			case 's':
				sval = va_arg(ap,char *);
				if (!gUseLogFile) kprintf("%s",sval);
				else fputs( sval, gTheFile);
				break;
			
			case 'c':
				schar = va_arg(ap,char);				
				if (!gUseLogFile) kprintf("%c",schar);
				else putc(schar, gTheFile);
				break;
				
			default:       
				if (!gUseLogFile) kprintf("%c",*p);
				else putc(*p, gTheFile);
				break;
		}
				 
	}
	va_end(ap);
}
   
Item	GetMyItem(void)
/* Get the item corresponding to the current task */
{
	return KernelBase->kb_CurrentTask->t.n_Item;
}

Boolean GetNextParam(int theArgCount, char** theArgList, int paramIndx, char** paramStr)
/* Get the next command line argument */
{
	char	*aStr;

	gInvalidArg = false;
	if ( paramIndx < (theArgCount - 1) )
	{
		aStr = theArgList[paramIndx + 1];
		if (aStr[0] != '-')
		{
			*paramStr = aStr;
			return true;
		}
	}
	
	return false;
}

Boolean isNumber(char *aNumStr)
{
	if (aNumStr[0] == '-')
	{
		if ( strlen(aNumStr) < 2 ) return false;
		aNumStr++;
	}
	return ( isdigit(aNumStr[0]) );	// just check first char for now
}

int32 HexToInt32(char *aString)
{
	int32	aInt32=0;
	char	aChar;
	
	for (; *aString; aString++)
	{
		aChar = tolower(*aString);
		if isdigit(aChar) aChar = aChar - '0';
		else if ( isxdigit( aChar ) ) aChar = aChar + 10 - 'a';
		aInt32= aInt32 * 16 + aChar;
	}
	
	return aInt32;
}

Boolean GetNextNumParam(int theArgCount, char** theArgList, int *paramIndx, char *paramName, long *aLong)
/* Get the next decimal numeric command line argument */
{
	char	*aNumStr;
	int		aIndex;

	if (strcasecmp(theArgList[*paramIndx], paramName) == 0)
	{
		aIndex = *paramIndx;
		if ( aIndex < (theArgCount - 1) ) // paramName is not the last argument
		{
			aNumStr = theArgList[aIndex + 1];
			if ( isNumber(aNumStr) )
			{
				*aLong = strtol(aNumStr,(char**)NULL,10);
				(*paramIndx)++;
				return true;
			}
		}
		gInvalidArg = true;
		return true;
	}
	
	return false;

}

Boolean isHexNumber(char *aNumStr)
{
	char	aChar;

	for (; *aNumStr; aNumStr++)
	{
		aChar = tolower(*aNumStr);
		if ( (!isxdigit(aChar)) )
			return false;
	}
	return true;
}

Boolean GetNextHexParam(int theArgCount, char** theArgList, int *paramIndx, long *aLong)
/* Get the next hex numeric command line argument */
{
	char	*aNumStr;
	int		aIndex;

	aIndex = *paramIndx;
	if ( aIndex < (theArgCount - 1) )
	{
		aNumStr = theArgList[aIndex + 1];
		if (*aNumStr == '0')
			if (aNumStr[1] == 'x'
				|| aNumStr[1] == 'X')
					aNumStr += 2;
		if ( isHexNumber(aNumStr) )
		{
			*aLong = (long) HexToInt32(aNumStr);
			(*paramIndx)++;
			return true;
		}
	}
	
	return false;
}

Boolean GetNextFlagParam(int theArgCount, char** theArgList, int* paramIndx, char* paramName, bool* aFlag)
/* Set a binary-valued tool option using the current command line arguments */
{
	char	*aStr;
	
	if (strcasecmp(theArgList[*paramIndx], paramName) == 0)
	{
		if ( GetNextParam(theArgCount, theArgList, *paramIndx, &aStr) )
		{
			if ( strcasecmp(aStr, kOn) == 0 )
				{
				*aFlag = true;
				(*paramIndx)++;
				}
			else if (strcasecmp(aStr, kOff) == 0)
				{
				*aFlag = false;
				(*paramIndx)++;
				}
			else if (strlen(aStr) != 0) gInvalidArg = true;
		}
		return true;
	}
	
	return false;
}

int32 SecondsToAudioTicks( int32 Secs )
/* Convert the specified number of seconds to ticks */
/* NOTE: You must open the audio folio to use this! */
{
	frac16 Rate;
	
	Rate = GetAudioRate() >> 16;
	return ( Secs * Rate );
}

int32 SleepSeconds( int32 Secs )
/* Sleep the specified number of seconds */
/* NOTE: You must open the audio folio to use this! */
{
	return SleepAudioTicks ( SecondsToAudioTicks( Secs ) );
}

