/* $Id: testMPEG.c,v 1.11 1994/11/02 01:48:30 spence Exp $
 *
 * testMPEG -- This program is used to run either of the two MPEG streamers
 * we have running today for the Cable project. One of the streamers is
 * avCableDigitalStreamer, which plays an MPEG1 stream over the network, the
 * other is avCablePESStreamer, which plays a PES MPEG stream over the network
 * or the file system.
 *
 * Some notes about the two streamers, applicable to Sept 26 1994
 *
 * avMPEGStreamer:
 * Plays Whitebook MPEG streams (each packet is 2324 bytes large) through
 * the normal 3DO file system.
 *
 * v0.6 *only* works on MIA2 cards. v0.7 (to be released Jul  1 1994) will work
 * on both MIA1 and MIA2 cards. MIA1 only works in DMA mode, MIA2 only works in
 * slipstream mode. By default, this program assumes MIA2. Use the -v option to
 * force video DMA mode for MIA1.
 *
 * avCableDigitalStreamer:
 * Plays separate audio and video PES MPEG streams.
 *
 * v0.1 *only* works on MIA2 cards. v0.7 (to be released Jul  1 1994) will work
 * on both MIA1 and MIA2 cards. MIA1 only works in DMA mode, MIA2 only works in
 * slipstream mode. By default, this program assumes MIA2. Use the -v option to
 * force video DMA mode for MIA1.
 *
 * If the filename passed with avOpenStreamFile() starts with a digit, then the
 * streamer assumes a file number is to be played accross the network. If not,
 * then the streamer assumes the name is that of a video file from a filesystem.
 *
 */

#include "types.h"
#include "av.h"
#include "event.h"
#include "time.h"
#include "debug.h"
#include "init3do.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "ctype.h"
#include "JoyPad.h"
#include "Open24BitGraphics.h"
#define PRIVILAGED_APP
#include "cam_services.h"

#define REPORT_ERR(x) {if ((x) < 0) PrintfSysErr(x);}
#define ABLE(x) abilities[(x)]

/*#define SHOWDEBUG*/

#ifdef SHOWDEBUG
#define DPRT(x) {kprintf(x);}
#else
#define DPRT(x)
#endif

/* This is the name if the SetTopTerminal configuration file, which contains
 * the IP numbers of the STT, the IIS, the SMU, and the destination Port
 * number.
 */
#define STT_FILENAME "/remote/system/stt/stt.config"

#define VOLUME_DELTA ((MAXDSPAMPLITUDE + 1) / (TOTAL_VOLUME_BARS / 2))

/*
 * parse_file()
 *
 * Parses the STT file.
 */

static char *parse_file( Stream *fp )
{
  static char buff[128];
  char *p, c;

  p = buff;
  buff[0] = 0;

  /* get the first character */
  if( !ReadDiskStream(fp, &c, 1) ) return( buff );

  while( c != '\r' && c != '#' ) {
    *p++ = c;
    if( !ReadDiskStream(fp, &c, 1) )
      c = '\n';
  }

  if( c == '#' )
    while( ReadDiskStream(fp, &c, 1) && c != '\r' )
      ;

  *p = 0; /* null terminate */
  return( buff );
}

void Usage()
{
  kprintf("Usage: either a filename,\n");
  kprintf("or -n <trackNumber>,\n");
  kprintf("-d [16|24] for Depth\n");
  kprintf("-s Screen allocated by this app, not the streamer\n");
  kprintf("-r seRverIP number\n");
  kprintf("-i Iis IP number\n");
  kprintf("-c Client IP number\n");
  kprintf("-p Port number\n");
  kprintf("-t sTreamer name\n");
  kprintf("-f Filename (will use avMPEGStreamer)\n");
  kprintf("-v force Video DMA\n");
  kprintf("-a use cAm instead of RPCs\n");
  kprintf("-b service ID for CAM\n");
  kprintf("-R disable RPCs and CAM - start movie by hand\n");
  kprintf("-U disable User Feedback\n");
  kprintf("\nA button moves on to the next numbered stream (Cable version only\n");
  kprintf("B button prints the stream name on screen and the current\n");
  kprintf("  stream position on the debug console\n");
  kprintf("C button closes the current stream and quits\n");
  kprintf("Left Shift displays debug bars\n");
  kprintf("Stop button stops the stream and closes it\n");
  kprintf("Pause button pauses/resumes the stream\n");
  kprintf("Up/Down changes volume\n");
  kprintf("Left/Right mutes/unmutes\n");
  CloseAVFolio();
  exit(0);
}

void PrintCOS(CAMOpenServiceReply *cosrep)
{
	kprintf("  num_CAMOSI        = %d\n",cosrep->num_CAMOSI);
	kprintf("  subServiceCount   = %d\n",cosrep->subServiceCount);
	kprintf("  currentServerTime = %d\n",cosrep->currentServerTime);
	kprintf("  serverTimeZone    = %d\n",cosrep->serverTimeZone);
	kprintf("  videoStartTime    = %d\n",cosrep->videoStartTime);
	kprintf("\n  CAMOSI [0x%x]\n",cosrep->CAMOSI);
	kprintf("  serviceAddress        = 0x%x:0x%x\n",
		cosrep->CAMOSI->serviceAddress.ipAddress,
		cosrep->CAMOSI->serviceAddress.port);
	kprintf("  protocol              = %d\n",cosrep->CAMOSI->protocol);
	kprintf("  serviceProgramID      = %d\n",cosrep->CAMOSI->serviceProgramID);
	kprintf("  serviceProgramVersion = %d\n",cosrep->CAMOSI->serviceProgramVersion);
	kprintf("\n  serviceInfo [0x%x]\n",cosrep->CAMOSI->serviceInfo);
	kprintf("  serviceID      = %d\n",cosrep->CAMOSI->serviceInfo->serviceID);
	kprintf("  serviceType    = %d\n",cosrep->CAMOSI->serviceInfo->serviceType);
	kprintf("  serviceVersion = %d\n",cosrep->CAMOSI->serviceInfo->serviceVersion);
	kprintf("  serviceName    = %s\n",cosrep->CAMOSI->serviceInfo->serviceName);
	kprintf("  serviceInfoFlg = 0x%x\n",cosrep->CAMOSI->serviceInfo->serviceInfoFlg);
}

CAMOpenServiceInfo *GetCAMOSI(CAMOpenServiceReply *cosrep)
{
	return(cosrep->CAMOSI);
}

/*********************************************************************/
extern int optind;
extern char *optarg;

int32 getopt(int32 nargc, char * const *nargv, const char *ostr );

int main(int argc, char *argv[])
{
	Err err;
	Item streamer;		/* The Item of the streamer used */
	Item stream;		/* The Item of the stream being played through the streamer */
	Item camFolio = 0;
	JoyPadState jp;
	char filenameBuild[32];
	char *filename;
	char *serverIP = "192.33.138.1", *iisIP = "192.33.138.2";
	char *clientIP = "192.33.138.57";
	char sttServer[16], sttiis[16], client[16];
	char *streamerName = AV_CABLE_DIGITAL_NAME;
	int32 track = -1;
	Item vbl;
	struct timeval tv = {5, 0};
	int32 i;
	int32 depth = 0;
	int32 port = 3001;
	Item endMsgPort;
	Item endMsg = 0;
	TagArg getAbilities[] =	/* What this streamer is capable of */
	{
		{AV_TAG_ABLE_PLAY, NULL},
		{AV_TAG_ABLE_PAUSE, NULL},
		{AV_TAG_ABLE_GETSTREAMPOS, NULL},
		{AV_TAG_ABLE_FFWD, NULL},
		{AV_TAG_ABLE_REW, NULL},
		{AV_TAG_ABLE_SETSTREAMPOS, NULL},
		{AV_TAG_ABLE_VOLUME, NULL},
		{AV_TAG_ABLE_MUTE, NULL},
		{TAG_END},
	};
	TagArg osf_tags[16];
	ScreenContext sc;
	avBitmapInfo avbi;
	Item screenGroupI = 0;
	Stream *fp;
	int32 volume = MAXDSPAMPLITUDE;

	CAMOpenServiceReq cosreq;
	CAMOpenServiceReply *cosrep = NULL;
	CAMOpenServiceInfo *camosi = NULL;
	int32 svc_id = 0;

	bool open = FALSE;
	bool loop = TRUE;
	bool pause = FALSE;
	bool quick = FALSE;
	bool allocateScreen = FALSE;
	bool abilities[16];
	bool atlanta = FALSE;
	bool forceDMA = FALSE;
	bool useRPC = TRUE;
	bool useCAM = FALSE;
	bool userFeedback = TRUE;

	/* Print out the version string */
	print_vinfo();

	if ((argc == 2) && (*argv[1] == '?'))
	{
		Usage();
		return(0);
	}

	filename = "1";

	if ((err = OpenAVFolio()) < 0)
	{
		PrintfSysErr(err);
		exit(err);
	}

	if ((err = OpenGraphicsFolio()) < 0)
	{
		CloseAVFolio();
		PrintfSysErr(err);
		exit(err);
	}

	vbl = GetVBLIOReq();

	/* Attempt to read the STT file. If this is not possible, then the default
	 * values as set above will be used (and these are set up for my
	 * configuration).
	 * These config values can be overriden from the shell using the appropriate
	 * command line parameters.
	 */
	fp = OpenDiskStream(STT_FILENAME, 0);
	if (fp)
	{
		strcpy(sttiis, parse_file(fp));
		iisIP = sttiis;
		strcpy(sttServer, parse_file(fp));
		serverIP = sttServer;
		strcpy(client, parse_file(fp));
		clientIP = client;
		port = strtoul(parse_file(fp), NULL, 10);
		printf("From stt.config:\n");
		printf("iis = %s\n", iisIP);
		printf("server = %s\n", serverIP);
		printf("client = 0x%lx\n", clientIP);
		printf("port = %ld\n", port);
		CloseDiskStream(fp);
	}
	else
	{
		printf("Error opening %s\n", STT_FILENAME);
	}

	/* Parse the command line */

	while ((i = getopt(argc, argv, "n:f:d:t:sr:c:i:p:vRUab:")) != -1) {
		switch (i) {
		case 'n':
		  /* Hmm. This looks like old code left over from the WhiteBookCD
		   * test code, where I am trying to build a filename of the 
		   * form "track8".
		   */
		  track = atoi(optarg); 
		  strcat(filenameBuild, "track");
		  strcat(filenameBuild, optarg);
		  filename = filenameBuild;
		  break;
		case 'f':
		  /* -f. This is the filename to use. */
		  filename = optarg;
		  /* use this streamer */
		  streamerName = "avMPEGStreamer";
		  break;
		case 'd':
		  depth = atoi(optarg);
		  if( depth != 16 && depth != 24 ) {
		    printf("ERROR:  depth must be either 16 or 24\n");
		    Usage();
		  }
		  break;
		case 't':
		  /* -t. Give the name of the streamer */
		  streamerName = optarg;
		  printf("Using streamer %s\n", streamerName);
		  break;
		case 's':
		  /* -s. I'm going to allocate the screen for the streamer to use
		   * rather than have the streamer allocate its own.
		   */
		  allocateScreen = TRUE;
		  break;
		case 'r':
		  /* -r. Override the serverIP */
		  serverIP = optarg;
		  kprintf("server IP = %s\n", serverIP);
		  break;
		case 'c':
		  /* -c. Override the clientIP (STT) */
		  clientIP = optarg;
		  kprintf("client IP = %s\n", clientIP);
		  break;
		case 'i':
		  /* -i. Override the IISIP */
		  iisIP = optarg;
		  kprintf("iis IP = %s\n", iisIP);
		  break;
		case 'p':
		  /* -p. Override the port number */
		  port = atoi(optarg);
		  kprintf("port = %ld\n", port);
		  break;
		case 'v':
		  /* -v. Force DMA, don't use Slipstream. Must be used if running
		   * on a MIA 1 card.
		   */
		  forceDMA = TRUE;
		  break;
		case 'R':
		  /* -R. Disable RPCs */
		  useRPC = FALSE;
		  useCAM = FALSE;
		  filename = "1-";	/* Temporary kludge. Name starts with a digit
					 * for network use, '-' to disable RPCs in the
					 * streamer.
					 */
		break;
		case 'U':
		  /* -U. Disable User feedback */
		  userFeedback = FALSE;
		  break;
		case 'a':
		  /* -a, Use CAM folio */
		  useCAM = TRUE;
		  useRPC = FALSE;
		  break;
		case 'b':
		  /* -b, service ID */
		  svc_id = atoi(optarg);
		  break;
		default:
		  Usage();
		}
	}

	/* Create a message port to be signalled when the stream has finished. */
	endMsgPort = CreateMsgPort(NULL, 0, 0);
	DPRT(("@@@ endMsgPort = 0x%lx\n", endMsgPort));

	/* Get the appropriate streamer */
	kprintf("Opening Cable streamer\n");
	streamer = avGetStreamer(streamerName);
	kprintf("streamer = 0x%lx\n", streamer);

	/* Were we succesful? */
	if (streamer > 0)
	{
		/* Get the abilities. This tells us what the streamer is capable
		 * of, so we know what control options to give the user.
		 *
		 * Each ta_Arg is a pointer to a bool variable, which the AVFolio
		 * will set TRUE or FALSE depending on whether the streamer
		 * supports the respective ability.
		 *
		 */
		for (i = 0; i < ((sizeof(getAbilities) / sizeof(TagArg)) - 1); i++)
		{
			getAbilities[i].ta_Arg = (void *)&abilities[i];
		}
		avGetStreamerAbilities(streamer, getAbilities);
		for (i = 0; i < (sizeof(getAbilities) / sizeof(TagArg)); i++)
		{
			DPRT(("Able %ld = %s\n", i, (abilities[i] ? "TRUE" : "FALSE")));
		}

		/* If we are allocating the screen, do so. */
		if (allocateScreen)
		{
			/* 16 bit NYI */
			screenGroupI = Open24BitGraphics(&sc, 2);
			if (screenGroupI > 0)
			{
				printf("@@@ Allocated screen\n");
				/* Set up the avBitmapInfo so we can tell the AVFolio
				 * what the screen and bitmaps are.
				 */
				avbi.avbi_bitmapVisible = 0;
				avbi.avbi_bitmapCount = 4;
				avbi.avbi_bpp = 24;
				for (i = 0; i < 4; i++)
				{
					avbi.avbi_bitmaps[i] = sc.sc_Bitmaps[i];
					avbi.avbi_ScreenI[i] = sc.sc_Screens[i];
				}
			}
			else
			{
				PrintfSysErr(screenGroupI);
				loop = FALSE;
			}
		}


		/* Here comes the main loop. */
		while (loop)
		{
			if ((!open) && (filename))
			{
				/* We need to open the file to be played. Start by building
				 * the tag list that is passed to avOpenStreamFile().
				 */

				int32 t = 0;
				if (screenGroupI > 0)
				{
					/* Here is the screen I just created */
					osf_tags[t].ta_Tag = AV_TAG_OSF_BITMAPS;
					osf_tags[t].ta_Arg = (void *)&avbi;
					t++;
				}

				if (useCAM)
				{
					camFolio = OpenCAMFolio(TRUE, NULL);
					if (camFolio > 0)
					{
						/* In here, we can theoretically talk to SeM. */
						cosreq.sessionType = SP_PLAYNOW_SETUP;
						cosreq.serviceID = svc_id;
						cosreq.serviceVers = 1;
						cosreq.objectID = track;
						cosreq.eventType = 1;
						cosreq.duration = 0;
						cosreq.orderID = 0;
		
						cosrep = CAMOpenService(&cosreq, 0);
						if (cosrep)
						{
							PrintCOS(cosrep);
							camosi = GetCAMOSI(cosrep);
							osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_CAMOSI;
							osf_tags[t].ta_Arg = (void *)camosi;
							t++;
						}
						else
						{
							kprintf("Could not open the CAM Service.\n");
							kprintf("QUAD Exit - I suggest you reboot the 3DO\n");
							exit(-1);
						}
					}
					else
					{
						useRPC = TRUE;
						kprintf("Could not open the CAM Folio. Using RPCs instead\n");
					}
				}

				if (useRPC)
				{
					/* Here's the IP information */
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_SERVERIP;
					osf_tags[t].ta_Arg = (void *)serverIP;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_CLIENTIP;
					osf_tags[t].ta_Arg = (void *)clientIP;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_PORT;
					osf_tags[t].ta_Arg = (void *)port;
					t++;
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_IISIP;
					osf_tags[t].ta_Arg = (void *)iisIP;
					t++;
					if (track != -1)
					{
						sprintf(filename, "%ld", track);
					}
				}

				kprintf("Opening file %s\n", filename);

				if (depth)
				{
					/* If we used -d to set a specific depth, build the
					 * tag here. If not, then the streamer will use its
					 * default depth.
					 */
					osf_tags[t].ta_Tag = ((depth == 16) ?  AV_TAG_OSF_DISPLAY16BPP : AV_TAG_OSF_DISPLAY24BPP);
					t++;
				}
				if (forceDMA)
				{
					/* If we used -v to force DMA mode, build the tag here. */
					osf_tags[t].ta_Tag = AV_TAG_OSF_CABLED_VDMA;
					t++;
				}

				osf_tags[t].ta_Tag = TAG_END;
				stream = avOpenStreamFile(streamer, filename, endMsgPort, osf_tags);
				REPORT_ERR(stream);

				if (open = (stream > 0))
				{
					/* Hooray! It opened. Now just play the stream. */

					err = avPlayStream(stream, AV_RATE_NORMAL, userFeedback);
					REPORT_ERR(err);
					quick = FALSE;

					/* Set the volume level */
					err = avStartUserFeedbackVA(stream, 
		    			                        AV_TAG_UF_VOLUME, volume,
		    			                        AV_TAG_UF_DURATION_FIELDS, 0,
		                			            TAG_END);
					REPORT_ERR(err);

				}
				else
				{
					/* Bummer dude. */
					loop = FALSE;
				}
			}
	
			if (stream > 0)
			{
				GetJoyPad(&jp, 1);

				/* Hmm. A nasty polling loop! What was I thinking here??? */
				if (endMsg = GetMsg(endMsgPort))
				{
					printf("@@@ End of the stream\n");
					ReplyMsg(endMsg, 0, NULL, 0);
					jp.xBtn = 1;	/* Force the CloseStream code */
				}

				if (jp.xBtn)
				{
					DPRT(("@@@ Closing stream\n"));
					err = avCloseStreamFile(stream);
					REPORT_ERR(err);
					open = FALSE;
					filename = NULL;
					stream = 0;
				}				
	
				/* Note that for volume and mute, we check that the streamer
				 * supports the action.
				 */
				if ((jp.upArrow) && (ABLE(6)))
				{
					DPRT(("@@@ Volume Up\n"));
					if (volume < MAXDSPAMPLITUDE)
					{
						volume += VOLUME_DELTA;
					}
					if (volume > MAXDSPAMPLITUDE)
					{
						volume = MAXDSPAMPLITUDE;
					}
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_VOLUME, volume,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}
	
				if ((jp.downArrow) && (ABLE(6)))
				{
					DPRT(("@@@ Volume Down\n"));
					if (volume > 0)
					{
						volume -= VOLUME_DELTA;
					}
					if (volume < 0)
					{
						volume = 0;
					}
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_VOLUME, volume,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}
	
				if ((jp.leftArrow) && (ABLE(7)))
				{
					/* Mute */
					DPRT(("@@@ mute on\n"));
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_MUTE, TRUE,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if ((jp.rightArrow) && (ABLE(7)))
				{
					/* Mute */
					DPRT(("@@@ mute off\n"));
					err = avStartUserFeedbackVA(stream,
		    			                        AV_TAG_UF_MUTE, FALSE,
		                			            TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if (jp.leftShift)
				{
					/* Temporary hack -- show the buffers on screen */
					err = avStartUserFeedbackVA(stream,
					      AV_TAG_LASTUF, NULL,
					      TAG_END, NULL);
					REPORT_ERR(err);
				}

				if ((jp.rightShift) && (ABLE(5)))
				{
					/* Jump forward five seconds. */
					DPRT(("@@@ forward 5 secs\n"));
					tv.tv_sec = 5;
					err = avSetStreamPositionVA(stream, AV_F_SETSTREAMPOS_DELTA, userFeedback,
					                           AV_TAG_SSP_ABSTIME, (void *)&tv,
					                           TAG_END);
					REPORT_ERR(err);
				}
	
				if ((jp.aBtn) && (track > 0))
				{
					/* Move on to the next numerically ordered stream. This only
					 * makes sense on the cable system where all the streams
					 * are numbers.
					 */
					err = avCloseStreamFile(stream);
					REPORT_ERR(err);
					track++;
					sprintf(filenameBuild, "track%ld", track);
					filename = filenameBuild;
					open = FALSE;
				}
	
				if (jp.bBtn)
				{
					struct timeval tv = {10, 0};
					avPoint avp = {150, 170, 0};

					/* Print the stream name on screen */
					err = avStartUserFeedbackVA(stream,
					        AV_TAG_UF_STREAMNAME, NULL,
					        AV_TAG_UF_POSITION, (void *)&avp,
					        AV_TAG_UF_DURATION_FIELDS, (void *)300,
					        TAG_END);
					if (err < 0)
					{
						PrintfSysErr(err);
					}
				}

				if ((jp.bBtn) && (ABLE(2)))
				{
					/* Get the current position of the stream */
					TagArg gsp[] =
					{
						{AV_TAG_GSP_FIELDS, NULL},
						{AV_TAG_GSP_ABSTIME, NULL},
						{AV_TAG_GSP_BYTES, NULL},
						{AV_TAG_GSP_BLOCKS, NULL},
						{TAG_END, NULL},
					};
					struct timeval *tv;
					avPoint avp = {40, 40, 0};

					DPRT(("@@@ Get position\n"));
					err = avStartUserFeedbackVA(stream,
					      AV_TAG_UF_TIME, NULL,
					      AV_TAG_UF_POSITION, &avp,
					      TAG_END, NULL);
					if (err < 0)
					{
						PrintfSysErr(err);
					}

					err = avGetStreamPosition(stream, gsp);
					if (err >= 0)
					{
						kprintf("Fields = %ld\n", gsp[0].ta_Arg);
						tv = (struct timeval *)gsp[1].ta_Arg;
						kprintf("Time  = %ld.%ld secs\n", tv->tv_sec, tv->tv_usec);
						kprintf("Bytes = %ld\n", gsp[2].ta_Arg);
						kprintf("Blocks = %ld\n", gsp[3].ta_Arg);
					}
					else
					{
						PrintfSysErr(err);
					}
				}

				if (jp.cBtn)
				{
					/* Close the stream */
					DPRT(("@@@ close stream\n"));
					if (stream > 0)
					{
						err = avCloseStreamFile(stream);
						REPORT_ERR(err);
					}
					open = FALSE;
					loop = FALSE;
				}
	
				if ((jp.startBtn) && (ABLE(1)))
				{
					/* Toggle between pause and resume, with user feedback */
					if ((pause) || (quick))
					{
						DPRT(("@@@ play normal\n"));
						err = avPlayStream(stream, AV_RATE_NORMAL, userFeedback);
						REPORT_ERR(err);
						quick = FALSE;
						pause = FALSE;
					}
					else
					{
						DPRT(("@@@ pause\n"));
						err = avPlayStream(stream, AV_RATE_PAUSE, userFeedback);
						REPORT_ERR(err);
						pause = TRUE;
					}
				}
			}
			else
			{
				PrintfSysErr(stream);
				loop = FALSE;
			}
			WaitVBL(vbl, 2);
		}

		/* All finished. We no longer need the streamer, so remove it from RAM. */
		DPRT(("@@@ remove streamer\n"));
		err = avRemoveStreamer(streamer);
		if (err < 0)
		{
			PrintfSysErr(err);
		}
	}
	else
	{
		PrintfSysErr(streamer);
	}

	if (screenGroupI > 0)
	{
		Close24BitGraphics(&sc, screenGroupI);
	}

	DeleteItem(vbl);
	CloseAVFolio();
	kprintf("See ya\n");
	exit(0);
}
