/*
	File:		ExamineRecords.c

	Contains:	routines for testing ExamineRecords() call

	Written by:	Eric Hegstrom

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 3/15/94	ELH		Created

	To Do:
*/

#include "TestSpec.h"
#include "database.h"
#include "globals.h"
#include "TSReporter.h"
#include "db_error.h"


// ---------------------------------------------
//
// Prints out the data contained in the fields passed in fields
//
void	PrintFields( Field	*fields[],	uint32	fieldCnt )
{
	int32	n;

	for (n = 1; n <= fieldCnt; n++) {						// iterate through all fields
		Report("  {%d}= ", (*fields)[n].FieldID);
		
		switch	((*fields)[n].DataType) {					// print out the data based on its type
//		switch	(FTYPE_STRING) {					// print out the data based on its type
		
			case (FTYPE_STRING):	Report("\"%s\"", (*fields)[n].Data.F_String);
									break;
									
			case (FTYPE_INT16):		Report("%d", (*fields)[n].Data.F_INT16);
									break;
									
			case (FTYPE_UINT16):	Report("%d", (*fields)[n].Data.F_UINT16);
									break;
									
			case (FTYPE_INT32):		Report("%d", (*fields)[n].Data.F_INT32);
									break;
									
			case (FTYPE_UINT32):	Report("%d", (*fields)[n].Data.F_UINT32);
									break;
		}
	}

	Report("\n");
}

// ---------------------------------------------
//
// Prints out the data contained in the fields passed in fields
//
void	PrintFieldTypes( Field	*fields[],	uint32	fieldCnt )
{
	int32	n;

	for (n = 1; n <= fieldCnt; n++) {						// iterate through all fields
		Report("  {%d}= ", (*fields)[n].FieldID);
		
		switch	((*fields)[n].DataType) {					// print out the data based on its type
		
			case (FTYPE_STRING):	Report("F_String");
									break;
									
			case (FTYPE_INT16):		Report("F_INT16");
									break;
									
			case (FTYPE_UINT16):	Report("F_UINT16");
									break;
									
			case (FTYPE_INT32):		Report("F_INT32");
									break;
									
			case (FTYPE_UINT32):	Report("F_UINT32");
									break;
									
			default:				Report("ID #%d", (*fields)[n].DataType);
		}
	}

	Report("\n");
}

// ---------------------------------------------
//
// Setsup the testing environment for the ExamineRecords() call
//   OPens the database and opens the records based on the appropriate query
//
Item	SetupExamineRecords( int32 indx )
{
	SearchQuery queryRec;

	curDBItem = OpenDataBase( TestSpec[indx].DataBaseName );
	if (DBDEBUG) Report( "OpenDataBase(%s) returns %x\n", TestSpec[indx].DataBaseName, curDBItem );
	if (curDBItem <= 0) return curDBItem;
	
	queryRec.SearchType = TestSpec[indx].SearchType;
	
	switch ( TestSpec[indx].SearchType ) {
				case (SQTYPE_BYLOGICAL):	queryRec.Form.QueryString = TestSpec[indx].Form.QueryString;
				
				
											Report("Query String: \"%s\"\n", TestSpec[indx].Form.QueryString);
				
											break;
											
				case (SQTYPE_BYOBJHANDLE):	queryRec.Form.ObjectHandle = TestSpec[indx].Form.ObjectHandle;
											break;
											
				case (SQTYPE_BYPRIMARYKEY):	queryRec.Form.PrimaryKey = TestSpec[indx].Form.PrimaryKey;
											break;
	}
	
	queryRec.SearchSet = (Handle) NULL;	
	
	curRecList = OpenRecords( curDBItem, &queryRec, TestSpec[indx].QueryFields, TestSpec[indx].NumQueryFields,
									&sizeCurRecList);
			
				
	if (curRecList <= 0 ){
		if (DBDEBUG) Report( "OpenRecords (dbItem = %x ) in Open Records FAILED err#%d\n", curDBItem, curRecList);		
		Report( "OpenRecords in Examine Records failed err#%d\n", curRecList);		
		PrintfSysErr (curRecList);
	}
	else {
		if (DBDEBUG) Report( "OpenRecords returns %x (dbItem = %x )(curRecList = %x);(%d Records Returned)\n", curRecList, curDBItem,curRecList, sizeCurRecList );		
	}
	return curRecList;

}

// ---------------------------------------------
//
//  Examines the records and checks the return result 
//  Prints out the values of the fields
//  currently no validation of the field results are done
//
Boolean	TestExamineRecords( int32 indx )
{
	uint32	n;
	int32	err;
	Field	TField[ 100 ];
	uint32	i;
	
	if (DBDEBUG) Report("About to ExamineRecords (curRecList = %x) (size = %x) \n", curRecList, sizeCurRecList );
	for (n=0; n < sizeCurRecList; n++ ) {

//		for (i=0; i < TestSpec[indx].NumQueryFields; i++) {
//			TField[i] = TestSpec[indx].QueryFields[i];
//		}


		err = ExamineRecord( curRecList, n, TestSpec[indx].QueryFields, TestSpec[indx].NumQueryFields );
		if (err == 0 ) {
//			PrintFieldTypes( (void *)(&(TestSpec[indx].QueryFields)), TestSpec[indx].NumQueryFields );
			if (DBDEBUG) Report( "REC #%d - ", n);
			if (DBDEBUG) PrintFields( (void *)(&(TestSpec[indx].QueryFields)), TestSpec[indx].NumQueryFields );
		}
		else {
			Report( "Error #%x Examining Record #%x\n",err, n);
			PrintfSysErr (err);
			return false;
		}
	}
	return true;
}

// ---------------------------------------------
//
// Cleans up after the examineRecordsTest ny closing the record and the database
//
Boolean	CleanupExamineRecords( void )
{
	Err		err, err2;

	err = CloseRecords( curRecList );
	if (DBDEBUG) Report("CloseRecords(%x) returns %x\n", curRecList, err );
	err2 = CloseDataBase( curDBItem );
	if (DBDEBUG) Report("CloseDataBase(%x) returns %x\n", curDBItem, err2 );
	
	return (err && err2);
}

