/*
	File:		LockRecords.c

	Contains:	routines for testing LockRecord()

	Written by:	Eric Hegstrom

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 3/15/94	ELH		Created

	To Do:
*/

#include "database.h"
#include "TestSpec.h"
#include "globals.h"
#include "TSReporter.h"


// ---------------------------------------------
Item	SetupLockRecords( int32 indx )
{
	SearchQuery queryRec;

	curDBItem = OpenDataBase( TestSpec[indx].DataBaseName );
	if (curDBItem <= 0) return curDBItem;
	
	queryRec.SearchType = TestSpec[indx].SearchType;
	
	switch ( TestSpec[indx].SearchType ) {
				case (SQTYPE_BYLOGICAL):	queryRec.Form.QueryString = TestSpec[indx].Form.QueryString;
											break;
											
				case (SQTYPE_BYOBJHANDLE):	queryRec.Form.ObjectHandle = TestSpec[indx].Form.ObjectHandle;
											break;
											
				case (SQTYPE_BYPRIMARYKEY):	queryRec.Form.PrimaryKey = TestSpec[indx].Form.PrimaryKey;
											break;
	}
	
	queryRec.SearchSet = (Handle) NULL;	
	curRecList = OpenRecords( curDBItem, &queryRec, TestSpec[indx].QueryFields, TestSpec[indx].NumQueryFields,
### MPW Shell - No match for file name pattern "TestSpec[indx].QueryFields,".

									&sizeCurRecList);
									
	return curRecList;

}

// ---------------------------------------------
Boolean	TestLockRecords( int32 indx )
{
	uint32	n;
	int32	err;
	int32	err2;
	
	for (n=1; n <= sizeCurRecList; n++ ) {

		err = LockRecord( curRecList, n );
		if (err != 1 ) {
			err2 = UnlockRecord( curRecList, n );
			if (err2 != 0){
				Report("Error #%d in Unlock Record\n", err2);
				return false;
			}
		}
		else if (err == 0){
			Report( "Failure Locking Record #%d\n", n);
			return false;
		}
		else {
			Report( "Error #%d Locking Record #%d\n", err, n);
			return false;
		}
	}
		
	return true;
}

// ---------------------------------------------
Boolean	CleanupLockRecords( void )
{
	Err		err, err2;

	err = CloseRecords( curRecList );
	err2 = CloseDataBase( curDBItem );
	
	return (err && err2);
}

