/*
	File:		ModifyRecords.c

	Contains:	

	Written by:	Eric Hegstrom

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 3/15/94	ELH		Created

	To Do:
*/

#include "database.h"
#include "TestSpec.h"
#include "globals.h"
#include "TSReporter.h"

// ----------  PROTOTYPES
Boolean	TestExamineRecords( int32 indx );



// ---------------------------------------------
void	PrintFields( Field	*fields[],	uint32	fieldCnt )
{
	int32	n;

	for (n = 1; n <= fieldCnt; n++) {
		Report("  {%d}= \"", (*fields)[n].FieldID);
		
		switch	((*fields)[n].DataType) {
		
			case (FTYPE_STRING):	Report("%s", (*fields)[n].Data.F_String);
									break;
									
			case (FTYPE_INT16):		Report("%d", (*fields)[n].Data.F_INT16);
									break;
									
			case (FTYPE_UINT16):	Report("%d", (*fields)[n].Data.F_UINT16);
									break;
									
			case (FTYPE_INT32):		Report("%d", (*fields)[n].Data.F_INT32);
									break;
									
			case (FTYPE_UINT32):	Report("%d", (*fields)[n].Data.F_UINT32);
									break;
		}
		Report("\"");
	}

	Report("\n");
}

// ---------------------------------------------
Item	SetupModifyRecords( int32 indx )
{
	SearchQuery queryRec;

	curDBItem = OpenDataBase( TestSpec[indx].DataBaseName );
	if (curDBItem <= 0) return curDBItem;
	
	queryRec.SearchType = TestSpec[indx].SearchType;
	
	switch ( TestSpec[indx].SearchType ) {
				case (SQTYPE_BYLOGICAL):	queryRec.Form.QueryString = TestSpec[indx].Form.QueryString;
											break;
											
				case (SQTYPE_BYOBJHANDLE):	queryRec.Form.ObjectHandle = TestSpec[indx].Form.ObjectHandle;
											break;
											
				case (SQTYPE_BYPRIMARYKEY):	queryRec.Form.PrimaryKey = TestSpec[indx].Form.PrimaryKey;
											break;
	}
	
	queryRec.SearchSet = (Handle) NULL;	

	curRecList = OpenRecords( curDBItem, &queryRec, TestSpec[indx].QueryFields, TestSpec[indx].NumQueryFields,
									&sizeCurRecList);
									
	return curRecList;

}

// ---------------------------------------------
Boolean	TestModifyRecords( int32 indx )
{
	uint32	n;
	int32	err;
	int32	err2;
	Boolean	bl;
	
	for (n=1; n <= sizeCurRecList; n++ ) {

		err = LockRecord( curRecList, n );
		if (err != 1 ) {
			err = ModifyRecord( curRecList, n, TestSpec[indx].ModifyFields, TestSpec[indx].NumModifyFields);
			err2 = UnlockRecord( curRecList, n );
			if( (err != 0) || (err2 != 0) ) {
				if (err != 0) Report("Error #%d in Modify Record\n", err);
				if (err2 != 0) Report("Error #%d in Unlock Record\n", err2);
				return false;
			}
		}
		else if (err == 0){
			Report( "Failure Locking Record #%d\n", n);
			return false;
		}
		else {
			Report( "Error #%d Locking Record #%d\n", err, n);
			return false;
		}
	}
	
	bl = TestExamineRecords( indx );				// lets examine them to check them out
	
	return true;
}

// ---------------------------------------------
Boolean	CleanupModifyRecords( void )
{
	Err		err, err2;

	err = CloseRecords( curRecList );
	if (DBDEBUG) ("CloseRecords(%x) returns %x\n", curRecList, err );
	err2 = CloseDataBase( curDBItem );
	if (DBDEBUG) ("CloseDataBase(%x) returns %x\n", curDBItem, err2 );
	
	return (err && err2);
}

