/*******************************************************************************************
 *	File:			Open24BitGraphics.c
 *
 *	Contains:		Screen init for 24-bit color.
 *
 *	Written by:		Stephen Landrum, George Mitsuoka, Neil Day, Steve Hayes and friends.
 *
 *	Copyright  1994 The 3DO Company. All Rights Reserved.
 *
 *	History:
 *  3/2/94			jhm		Added "#define NFW 0" so the compiler won't generate
 *							warnings while regenerating make file dependencies.
 *	2/24/94			bck		Fixed text overlay for 24 bit display.
 *	2/24/94			seh		Add text overlay.
 *	2/20/94			seh		Double buffering.
 *	2/14/94			nmd		Added 24 bit frame buffer support for single buffer
 *	?/?/??			gm		created from Landrum's example
 *
 *******************************************************************************************/

#include "Portfolio.h"
#include "event.h"
#include "filestream.h"
#include "filestreamfunctions.h"
#include "filefunctions.h"
#include "Parse3DO.h"
#include "Form3DO.h"
#include "Utils3DO.h"

#include "stdlib.h"
#include "graphics.h"	/* for CCB */

//#include "world.h"
#include "Open24BitGraphics.h"

#define MAXVDLS		2
#define NVDLS		2
#define NBUFFERS 	(NVDLS*2)

#define	prt(x)			printf x
#define catch_nil(v,x)	if( !v ) { prt(x); goto cleanup; }
#define catch_neg(v,x)	if(v<0) { prt(x);PrintfSysErr((Item)v); goto cleanup; }
#define	dbg(x)			prt(x)

///************************************************************************************
///	GLOBALS
///************************************************************************************

/* Displaying a screen within a task is a three level process
	1) create a screen group
	2) add the screen groups to the graphics folios
	3) display a screen from a group 
*/

#if 0
//Item 	ScreenItems[NBUFFERS];
Item	ScreenGroupItem;
//Item	BitmapItems[NBUFFERS];
//Bitmap	*Bitmaps[NBUFFERS];
#endif
/* Stuff we need 'cause this is not well thought out */
uint32 	bmw[NBUFFERS]={320,320,320,320};// ,320,320}; 
uint32 	bmh[NBUFFERS]={240,240,240,240};// ,240,240};
ubyte 	*bmbufs[NBUFFERS];
ubyte	*bmbuffer[NVDLS] = {NULL};

int32	*vdlarray[NBUFFERS];
int32	*vdl[NVDLS];

TagArg 	taScreenTags[] =
			{
			  CSG_TAG_SPORTBITS,	(void*)0,
//			  CSG_TAG_VDLPTR_ARRAY,	(void*)0,
			  CSG_TAG_DISPLAYHEIGHT,(void*)240,
			  CSG_TAG_SCREENCOUNT,	(void*)NBUFFERS,
			  CSG_TAG_SCREENHEIGHT,	(void*)240,
			  CSG_TAG_BITMAPCOUNT,	(void*)1,
			  CSG_TAG_BITMAPWIDTH_ARRAY,	(void*)bmw,
			  CSG_TAG_BITMAPHEIGHT_ARRAY,	(void*)bmh,
			  CSG_TAG_BITMAPBUF_ARRAY,	(void*)bmbufs,
			  CSG_TAG_DONE,		(void*)0,
			};

///************************************************************************************
///	PROTOTYPES
///************************************************************************************

void	initVDL( int32 *vdl, int32 bitmapindex );

/*************************************************************************************
	OpenGL
	
	initialize and
*************************************************************************************/
//uint32 OpenGL(char **buffer)
int32 Open24BitGraphics (ScreenContext * sc, int nScreens)
{
	long 	i,v;
	int32	ret;
	Item	ScreenGroupItem;
	int32	*tmpvdl;
	Item	vramioreq;

	sc->sc_nScreens = nScreens;
	
	if (nScreens != NVDLS)
		printf("ERROR: I only know how to open two 24bit screens!\n");
		
	if (OpenGraphicsFolio())
		{
		DIAGNOSTIC("Cannot open graphics folio");
		return FALSE;
		}

	/* taScreenTags[0].ta_Arg = (void *)GETBANKBITS( GrafBase->gf_ZeroPage );
	 * -- Just leave SPORTBITS as 0 so that *any* bank will do. SAS
	 */

	for (v=0; v<NVDLS; v++) {
		vdl[v] = (int32 *)AllocMem(6*40*4, MEMTYPE_DMA | MEMTYPE_VRAM | MEMTYPE_STARTPAGE);
			catch_nil(vdl[v],("failed to allocate vdl %d\n",v));
		vdlarray[v*2] = vdl[v];
		vdlarray[(v*2)+1] = vdl[v] + 120;
	}
		
	sc->sc_nFrameBufferPages = (bmw[0]*bmh[0]*4+GrafBase->gf_VRAMPageSize-1) / GrafBase->gf_VRAMPageSize;
	
	for (v=0; v<NVDLS; v++ ) {
		bmbuffer[v] = (ubyte *)ALLOCMEM((int)(sc->sc_nFrameBufferPages*GrafBase->gf_VRAMPageSize),
							            MEMTYPE_STARTPAGE|MEMTYPE_VRAM|MEMTYPE_CEL);
		catch_nil(bmbuffer[v],("failed to allocate screen memory\n"));
		bmbufs[v*2] = bmbuffer[v];
		bmbufs[(v*2)+1] = bmbuffer[v];
	}

//	taScreenTags[1].ta_Arg = (void *)vdlarray;

	ScreenGroupItem = CreateScreenGroup (&(sc->sc_Screens[0]), taScreenTags);
		catch_neg(ScreenGroupItem,("Failed to CreateScreenGroup\n"));

	AddScreenGroup (ScreenGroupItem, NULL);
	
	for (i=0; i<NBUFFERS; i++) {
		Screen *screen;
		
		screen = (Screen*) LookupItem (sc->sc_Screens[i]);
			catch_nil(screen ,("Failed to Lookup ScreenItem\n"));
		sc->sc_BitmapItems[i] = screen->scr_TempBitmap->bm.n_Item;
		sc->sc_Bitmaps[i] = screen->scr_TempBitmap;
	}
	
	for (v=0; v<NVDLS; v++ ) {
		tmpvdl = vdl[v];
		initVDL (tmpvdl, v*2);
		initVDL (tmpvdl+120, (v*2)+1);
		tmpvdl[83] = (int32)(tmpvdl+120);
		tmpvdl[123] = (int32)GrafBase->gf_VDLPostDisplay;
	
		i = SubmitVDL ((VDLEntry*)tmpvdl, 30*4, VDLTYPE_FULL);
		if (i<0) {
			dbg (("Error in SubmitVDL  (%lx)\n", i));
			return (-1);
		}
		SetVDL (sc->sc_Screens[v*2], i);
	
		i = SubmitVDL ((VDLEntry*)tmpvdl+30*4, 30*4, VDLTYPE_FULL);
		if (i<0) {
			dbg (("Error in SubmitVDL  (%lx)\n", i));
			return (-1);
		}
		SetVDL (sc->sc_Screens[(v*2)+1], i);
	}
	
	vramioreq = GetVRAMIOReq();

	SetVRAMPages (vramioreq, sc->sc_Bitmaps[0]->bm_Buffer, 0, sc->sc_nFrameBufferPages, ~0L);
	SetVRAMPages (vramioreq, sc->sc_Bitmaps[2]->bm_Buffer, 0, sc->sc_nFrameBufferPages, ~0L);
	
	// TBD don't I have to free the vramioreq?
	
	ret = DisplayScreen (sc->sc_Screens[0], sc->sc_Screens[1]);
		catch_neg(ret,("displayscreen failed\n"));
		
	for (v=0; v<NVDLS; v++ ) {
		prt(("VideoBuffer[%d] $%x\n",v, (uint32)sc->sc_Bitmaps[v*2]->bm_Buffer));
	}
	
	sc->sc_nFrameByteCount = sc->sc_nFrameBufferPages * GrafBase->gf_VRAMPageSize;
	sc->sc_curScreen = 0;

	return ScreenGroupItem;
cleanup:
	return -1;
}

/*************************************************************************************
	initVDL
*************************************************************************************/

void initVDL (int32 *vdl, int32 bitmapindex)
{
	int32 j;

	vdl[0] = VDL_ENVIDDMA|VDL_LDCUR|VDL_LDPREV|(34<<VDL_LEN_SHIFT)|VDL_PREVSEL
			|(240<<VDL_LINE_SHIFT)|VDL_DISPMOD_320|VDL_480RES;
	vdl[1] = ((uint32)bmbufs[bitmapindex]);
	vdl[2] = ((uint32)bmbufs[bitmapindex]) + 2;
	vdl[3] = (int32)(vdl+40);
	
	for (j=0; j<8; j++)
	{
		vdl[4+j] = (j<<24)|(j*2*0x00010101);
	}
	for (j=8; j<16; j++)
	{
		vdl[4+j] = (j<<24)|((j+j+225)*0x00010101);
	}
	for (j=16; j<32; j++)
	{
		vdl[4+j] = (j<<24)|(((31-j)*16)*0x00010101);
	}
	vdl[4+32] = VDL_DISPCTRL|VDL_BACKGROUND|0  | VDL_CLUTBYPASSEN;
	vdl[4+33] = VDL_DISPCTRL| VDL_CLUTBYPASSEN |VDL_VINTEN|VDL_HINTEN|VDL_HSUB_ZERO|VDL_BLSB_BLUE
				|(int32)((bitmapindex&1)?VDL_VSUB_ONE:VDL_VSUB_ZERO);
	for (j=4+34; j<40; j++)
	{
		vdl[j] = VDL_NULLVDL;
	}

	vdl[40] = (34<<VDL_LEN_SHIFT)|(0<<VDL_LINE_SHIFT)|VDL_DISPMOD_320|VDL_480RES;
	vdl[41] = ((uint32)bmbufs[bitmapindex]);
	vdl[42] = ((uint32)bmbufs[bitmapindex]) + 2;
	vdl[43] = (int32)(vdl+80);
	for (j=0; j<8; j++)
	{
		vdl[44+j] = (j<<24)|(j*2*0x00010101);
	}
	for (j=8; j<16; j++)
	{
		vdl[44+j] = (j<<24)|((j+j+225)*0x00010101);
	}
	for (j=16; j<32; j++)
	{
		vdl[44+j] = (j<<24)|(((31-j)*16)*0x00010101);
	}
	vdl[44+32] = VDL_DISPCTRL|VDL_BACKGROUND|0 | VDL_CLUTBYPASSEN;
	vdl[44+33] = VDL_DISPCTRL| VDL_CLUTBYPASSEN | VDL_VINTEN|VDL_HINTEN|VDL_HSUB_ZERO|VDL_BLSB_BLUE
				|(int32)((bitmapindex&1)?VDL_VSUB_ONE:VDL_VSUB_ZERO);
	for (j=44+34; j<40+40; j++)
	{
		vdl[j] = VDL_NULLVDL;
	}

	vdl[80] = (34<<VDL_LEN_SHIFT)|(17<<VDL_LINE_SHIFT)|VDL_DISPMOD_320|VDL_480RES;
	vdl[81] = ((uint32)bmbufs[bitmapindex]);
	vdl[82] = ((uint32)bmbufs[bitmapindex]) + 2;
	vdl[83] = (int32)GrafBase->gf_VDLPostDisplay;
	for (j=0; j<8; j++)
	{
		vdl[84+j] = (j<<24)|(j*2*0x00010101);
	}
	for (j=8; j<16; j++)
	{
		vdl[84+j] = (j<<24)|((j+j+225)*0x00010101);
	}
	for (j=16; j<32; j++)
	{
		vdl[84+j] = (j<<24)|(((31-j)*16)*0x00010101);
	}
	vdl[84+32] = VDL_DISPCTRL|VDL_BACKGROUND|0 | VDL_CLUTBYPASSEN;
	vdl[84+33] = VDL_DISPCTRL| VDL_CLUTBYPASSEN | VDL_VINTEN|VDL_HINTEN|VDL_HSUB_ZERO|VDL_BLSB_BLUE
				|(int32)((bitmapindex&1)?VDL_VSUB_ONE:VDL_VSUB_ZERO);
	for (j=84+34; j<80+40; j++)
	{
		vdl[j] = VDL_NULLVDL;
	}
}


/***********************************************************************************************
 *
 * void Close24BitGraphics() added by SAS Apr 23 1994 
 *
 **********************************************************************************************/

void Close24BitGraphics(ScreenContext *sc, Item screenGroupItem)
{
	int32 i;
	Err err;

	if (screenGroupItem)
	{
		err = RemoveScreenGroup(screenGroupItem);
		if (err < 0)
		{
			PrintfSysErr(err);
		}
	
		err = DeleteScreenGroup(screenGroupItem);
		if (err < 0)
		{
			PrintfSysErr(err);
		}
	
		for (i = (NVDLS - 1); i >= 0; i--)
		{
			if (bmbuffer[i])
			{
				FreeMem(bmbuffer[i], (int)(sc->sc_nFrameBufferPages*GrafBase->gf_VRAMPageSize));
			}
			if (vdl[i])
			{
				FreeMem(vdl[i], 6*40*4);
			}
		}
	}

	return;
}
