

/*
	File:		TSReporter.c

	Contains:	Reporter utilities for test Shell

	Written by:	Eric Hegstrom

	Copyright:	 1994 by The 3DO Company. All rights reserved.
				This material constitutes confidential and proprietary
				information of the 3DO Company and shall not be used by
				any Person or for any purpose except as expressly
				authorized in writing by the 3DO Company.

	Change History (most recent first):

		 <1>	 2/28/94	ELH		Created

	To Do:
*/

#include "TSReporter.h"
#include "stdio.h"

// -------------- reporter globals

FILE		*gTSRptLogFile;						// log file pointer
Boolean		TSRptUseScreenFile = true;
Boolean		TSRptUseLogFile ;


void 	TSprintd(int32 n, FILE *theFile)
/* Used by pipedata to output decimal values to the log file */
{
	if (n/10)
		TSprintd(n/10,theFile);
	 putc((char) n%10 + '0',theFile);
}


void 	TSprintHex(int32 n, FILE *theFile)
/* Used by pipedata to output hex values to the log file */
{
	long	indx;
	char	*hexChar[] = {	"0","1","2","3","4",
							"5","6","7","8","9",
							"a","b","c","d","e","f" };
	
	for (indx = 7; indx >= 0; indx--)
	{
		fputs( hexChar[((n) >> indx * 4) & 0xF], theFile);	
	}
}


void vPipeData( char *formatStr, va_list ap)
{
	char		*p,*sval,schar;
	int32		ival;
	
	for ( p = formatStr ; *p ; p++ )
	{
		if (*p == 0x0A) // look explicitly for a newline hex 10 or '\n'
		{
			if (!TSRptUseLogFile) kprintf("\n");
			else putc(0x0D, gTSRptLogFile);
			
			continue;
		}
		if (*p != '%')
		{
			if (!TSRptUseLogFile) kprintf("%c",*p);
			else putc(*p, gTSRptLogFile);
			
			continue;
		}
		switch(*++p)
		{
			case 'd':
			case 'i':
			case 'u':
				ival = va_arg(ap, int32);
				if (!TSRptUseLogFile) kprintf("%i",ival);
				else TSprintd(ival,gTSRptLogFile);
				break;
				
			case 'x':
				ival = va_arg(ap, int32);
				if (!TSRptUseLogFile) kprintf("%08lx",ival);
				else TSprintHex(ival,gTSRptLogFile);
				break;
				
			case 's':
				sval = va_arg(ap,char *);
				if (!TSRptUseLogFile) kprintf("%s",sval);
				else fputs( sval, gTSRptLogFile);
				break;
			
			case 'c':
				schar = va_arg(ap,char);				
				if (!TSRptUseLogFile) kprintf("%c",schar);
				else putc(schar, gTSRptLogFile);
				break;
				
			default:       
				if (!TSRptUseLogFile) kprintf("%c",*p);
				else putc(*p, gTSRptLogFile);
				break;
		}
				 
	}
}
 
// ====================================================
void TSRptvPipeScreen( char *formatStr, va_list ap)
{
	TSRptUseLogFile = false;

	vPipeData( formatStr, ap );
}

// ====================================================
void TSRptvPipeLog( char *formatStr, va_list ap)
{	
	TSRptUseLogFile = true;

	vPipeData( formatStr, ap );
}

// ====================================================
void 	TSRptItoa(int32 n, char *s)
/* atoi func */
{
	int32	k;
	int32	p = 4;
	
	strcpy(s, "00000\0");

	while ((n > 0)&&(p > 0)) {
		k = n/10;
		s[p--] = ((n-(k*10)) +'0');
		n = k;
	}
}

// -----------------   REPORTER EXTERNALY AVAILABLE CALLS   ------------------

// ====================================================
// Initialize the reporter system and set the desired verbosity level
void ReportInit( Boolean useScreen)
{
	int32	vNum = 0;
	char 	vStr[5];
	char 	fName[128];
	
	
	if (useScreen) TSRptUseScreenFile = true;
	else TSRptUseScreenFile = false;
	do { 
		vNum++;
		TSRptItoa( vNum, vStr );
		strcpy( fName, fNameBase );
		strcat( fName, vStr );
		gTSRptLogFile  = fopen(fName,"r");
		if (gTSRptLogFile != NULL) fclose(gTSRptLogFile);
	} while ( gTSRptLogFile != NULL);
	
	gTSRptLogFile  = fopen(fName,"w");			// open unique file
	kprintf("Opening Log File %s\n", fName);
	if (gTSRptLogFile == NULL) kprintf("Error Opening Log File %s\n", fName);
}

// ====================================================
// Terminates Reporter
void ReportTerm( void )
{
	if (gTSRptLogFile != NULL) fclose(gTSRptLogFile);
}

// ====================================================
void Report( char *formatStr, ...)
{
	va_list	ap;
	
	va_start(ap, formatStr);
	TSRptvPipeLog(  formatStr, ap  );
	
	if (TSRptUseScreenFile ) {
		TSRptvPipeScreen( formatStr, ap );
		va_end( ap );
	}
}

