/*******************************************************************************************
 *	File:			PlayImageStream.h
 *
 *	Contains:		definitions for high level stream playback function
 *
 *	Written by:		Joe Buczek
 *
 *	Copyright  1993 The 3DO Company. All Rights Reserved.
 *
 *	History:
 *	11/30/93		jb		Modified into a Join/Image stream player
 *	10/25/93		jb		Add userContext to user function calling sequence
 *	10/14/93		jb		New today
 *
 *******************************************************************************************/

#ifndef	_PLAYSTREAM_H_
#define	_PLAYSTREAM_H_

#ifndef __GRAPHICS_H
#include "Graphics.h"
#endif

#ifndef _TYPES_H
#include "Types.h"
#endif

#ifndef	_PREPARESTREAM_H_
#include "PrepareStream.h"
#endif

#include "DataAcq.h"
#include "SAudioSubscriber.h"
#if 0
	#include "CPakSubscriber.h"
#endif
#include "ControlSubscriber.h"
#include "JoinSubscriber.h"
#include "DataStreamDebug.h"

#define	IMAGE_DATA_TYPE		((unsigned long) 'IMAG')

typedef long (*PlayerUserFn)( void *ctx, void *userContext );

/**************************************/
/* Internal player context descriptor */
/**************************************/
typedef struct Player {

	PlayerUserFn		userFn;				/* user callback function pointer */
	void				*userContext;		/* for use by user callback function */

	DSHeaderChunk		hdr;				/* Copy of stream header from stream file */

	DSDataBufPtr		bufferList;			/* ptr to linked list of buffers used by streamer */
	AcqContextPtr		acqContext;			/* ptr to data acquisition thread's context */
	DSStreamCBPtr		streamCBPtr;		/* ptr to stream thread's context */

	ScreenContext		*screenContextPtr;	/* for drawing to the screen */
	Item				VBLIOReq;			/* ioreq for calling WaitVBL() */
	Item				VRAMIOReq;			/* ioreq for calling CopyVRAMPages() */
	Item				messagePort;		/* port for receiving end of stream message */
	Item				messageItem;		/* msg item for sending streamer requests */
	Item				endOfStreamMessageItem; /* msg item that is replied to as end of stream */

	CtrlContextPtr		controlContextPtr;	/* Control subscriber context ptr */

	SAudioContextPtr	audioContextPtr;	/* Audio subscriber context ptr */

	JoinContextPtr		joinContextPtr;		/* Join subscriber context ptr */
	Item				joinPort;			/* join subscription port */

#if 0
	CPakContextPtr		cpakContextPtr;		/* Cinepak subscriber context ptr */
	CPakRecPtr			cpakChannelPtr;		/* Cinepak subscriber channel ptr */
#endif

	} Player, *PlayerPtr;


/*****************************/
/* Public routine prototypes */
/*****************************/

int32	PlayImageStream( ScreenContext *screenContextPtr, char* streamFileName, 
							PlayerUserFn userFn, void *userContext );
void	DismantlePlayer( PlayerPtr ctx );

#endif	/* _PLAYSTREAM_H_ */
