/*****

$Id: ls.c,v 1.11 1994/05/05 20:08:39 martin Exp $

$Log: ls.c,v $
 * Revision 1.11  1994/05/05  20:08:39  martin
 * now only includes necessary include files
 * uses CreateIOReq() instead of CreateItem() to create IOReqs
 * improved formatting
 *
 * Revision 1.10  1993/05/28  21:43:11  dplatt
 * Cardinal3 changes, get ready for Dragon
 *
 * Revision 1.9  1993/03/16  06:41:42  dplatt
 * SysErr()-> PrintfSysErr()
 *
 * Revision 1.8  1993/03/16  06:36:37  dplatt
 * Functional Freeze release
 *
 * Revision 1.7  1992/12/08  05:59:52  dplatt
 * Magenta changes
 *
 * Revision 1.6  1992/11/06  01:12:39  dplatt
 * Make LoadProgram() into a library routine
 *
 * Revision 1.5  1992/10/24  00:40:56  dplatt
 * Bluebird changes and bug fixes
 *
 * Revision 1.4  1992/10/16  01:22:24  dplatt
 * First cut at bluebird release changes
 *
 * Revision 1.3  1992/10/01  23:36:21  dplatt
 * Switch to int32/uint32
 *
 * Revision 1.2  1992/09/11  22:37:38  dplatt
 * New compiler
 *
 * Revision 1.1  1992/09/11  00:42:34  dplatt
 * Initial revision
 *

 *****/

/*
  Copyright The 3DO Company, Inc., 1991-1994.
  All Rights Reserved Worldwide
*/

/*
  ls.c - lists the contents of a directory
*/

#include "types.h"
#include "io.h"
#include "discdata.h"
#include "operror.h"
#include "strings.h"
#include "stdio.h"

#include "filesystem.h"
#include "filefunctions.h"
#include "directory.h"
#include "directoryfunctions.h"

/*
   Debug code to print entries of an Opera directory.
*/

void ListDirectoryContents(Item openFileItem)
{
  Directory *dir;
  DirectoryEntry de;
  Item ioReqItem;
  int32 entry;
  int32 err;
  char truePath[512];
  IOInfo ioInfo;
  IOReq *ioReq;

  ioReqItem = CreateIOReq( NULL, 0, openFileItem, 0 );
  if (ioReqItem < 0) {
    printf("Couldn't create IOReq for open-file item %d\n", openFileItem);
    return;
  }
  ioReq = (IOReq *) LookupItem(ioReqItem);
  memset(&ioInfo, 0, sizeof ioInfo);
  ioInfo.ioi_Command = FILECMD_GETPATH;
  ioInfo.ioi_Recv.iob_Buffer = truePath;
  ioInfo.ioi_Recv.iob_Len = sizeof truePath;
  err = DoIO(ioReqItem, &ioInfo);
  if (err < 0 || (err = ioReq->io_Error) < 0) {
    PrintfSysErr(err);
    return;
  }
  dir = OpenDirectoryItem(openFileItem);
  if (!dir) {
    printf("Could not open a directory for %s\n", truePath);
    return;
  }
  printf("\nContents of directory %s:\n\n", truePath);
  entry = 1;
  while (1) {
    err = ReadDirectory(dir, &de);
    if (err < 0) break;
    printf("%5s", (char *) &de.de_Type);
    printf(" %8d", de.de_ByteCount);
    printf(" %4d*%4d", de.de_BlockCount, de.de_BlockSize);
    printf(" %3d", de.de_AvatarCount);
    printf(" %8x", de.de_Flags);
    printf(" %s\n", de.de_FileName);
    entry ++;
  }
  CloseDirectory(dir);
  printf("\nEnd of directory\n");
  printf("\n");
}


int32 ListDirectory (char *path)
{
  Item rootItem;
  rootItem = OpenDiskFile(path);
  if (rootItem < 0) {
    printf("Could not open %s: \n", path);
    PrintfSysErr(rootItem);
    printf("\n");
    return -999;
  }
  ListDirectoryContents(rootItem);
  CloseDiskFile(rootItem);
  return 0;
}

int main (int32 argc, char **argv)
{
  int32 i;
  if (argc <= 1) {
    ListDirectory(".");
  } else {
    for (i = 1 ; i < argc; i++) {
      ListDirectory(argv[i]);
    }
  }
  return 0;
}

