/* $Id: memdebugexample.c,v 1.4 1994/06/14 22:18:16 vertex Exp $ */

#include "types.h"
#include "stdio.h"

#define MEMDEBUG
#include "mem.h"


/*****************************************************************************/


int main(int argc, char **argv)
{
void *mem;
uint32 *ptr;

    if (CreateMemDebug(MEMDEBUGF_PAD_COOKIES |
                       MEMDEBUGF_ALLOC_PATTERNS |
                       MEMDEBUGF_FREE_PATTERNS |
                       MEMDEBUGF_CHECK_ALLOC_FAILURES |
                       MEMDEBUGF_KEEP_TASK_DATA,NULL) >= 0)
    {
        /* test what happens when you allocate nothing */
        AllocMem(0,0);

        /* test what happens when we ask for too much */
        AllocMem(123456789,0);

        /* test a NULL pointer */
        FreeMem(NULL,0);

        /* should say there's nothing to say... */
        DumpMemDebug(NULL);

        /* test a bogus pointer */
        FreeMem((void *)1,0);

        /* check a free operation with the wrong size */
        mem = AllocMem(123,MEMTYPE_ANY);
        FreeMem(mem,234);

        /* check if DumpMemDebug() will list these correctly */
        AllocMem(123,MEMTYPE_ANY);
        AllocMem(234,MEMTYPE_ANY);
        AllocMem(345,MEMTYPE_ANY);
        AllocMem(456,MEMTYPE_ANY);

        /* trash the cookie before the allocation and see if it is noticed */
        ptr = (uint32 *)AllocMem(333,MEMTYPE_ANY);
        ptr--;
        *ptr = 0;
        ptr++;

        /* see if the trashed cookie is detected */
        SanityCheckMemDebug(NULL);

        /* the trashed cookie should be reported once again by this call */
        FreeMem(ptr,333);

        /* output everything still allocated */
        DumpMemDebug(NULL);

        DeleteMemDebug();
    }
}
